<?php

use UmiCms\Service;

/**
 * Created by Evgenii Ioffe
 * @author Evgenii Ioffe <ioffe@umispec.ru>
 * @copyright Copyright (c) 2019, Evgenii Ioffe
 */
class OfdFermaAdmin
{
    use baseModuleAdmin;
    use tModulePart;

    /**
     * @var $module ofdFerma
     */
    public $module;

    public function dashboard()
    {
        $this->module->redirect('/admin/ofdFerma/settings/');
    }

    public function receipts()
    {
        if ($this->getModule()->ifNotXmlMode()) {
            if ($this->module->ifNotJsonMode()) {
                Service::Session()->set('filter', getRequest('filter'));
            }
            $this->setDataSetDirectCallMessage();
            return;
        }

        $limit = $this->getLimit();
        $offset = $this->getOffset($limit);

        $sel = new selector('objects');
        $sel->types('hierarchy-type')->name(OfdFermaReceiptModel::module, OfdFermaReceiptModel::method);

        $filter = Service::Session()->get('filter');
        if ($filter) {
            foreach ($filter as $field => $condition) {
                if (!$condition) {
                    continue;
                }

                try {
                    if ($field == OfdFermaReceiptModel::field_order) {
                        $selOrder = new selector('objects');
                        $selOrder->types('object-type')->name('emarket', 'order');
                        $selOrder->where('number')->equals($condition);
                        $selOrder->option('return', 'id');

                        $arOrderId = array();
                        foreach ($selOrder as $row) {
                            $id = getArrayKey($row, 'id');
                            if (!$id) {
                                continue;
                            }

                            $arOrderId[] = $id;
                        }

                        if (count($arOrderId)) {
                            $sel->where($field)->equals($arOrderId);
                        }
                    } else {
                        if (is_array($condition) && isset($condition['date'])) {
                            $from = trim(getArrayKey($condition['date'], 'from'));
                            $to = trim(getArrayKey($condition['date'], 'to'));
                            if ($from && $to) {
                                $sel->where($field)->between(strtotime($from), strtotime($to));
                            }
                        } else {
                            $sel->where($field)->equals($condition);
                        }
                    }
                } catch (Exception $e) {
                }
            }
        }

        $sel->order('id')->desc();
        $sel->limit($offset, $limit);
        selectorHelper::detectFilters($sel);

        $this->setDataType('list');
        $this->setActionType('view');
        $settingsList = $this->prepareData($sel->result(), 'objects');
        $this->setData($settingsList, $sel->length());
        $this->setDataRangeByPerPage($limit, $this->getCurrentPage());
        $this->doData();
    }

    public function settings()
    {
        $object = $this->module->getOfdFermaSettingsObject();
        if (!$object instanceof umiObject) {
            $umiObjectsCollection = umiObjectsCollection::getInstance();
            $objectId = $umiObjectsCollection->addObject('Настройки', umiObjectTypesCollection::getInstance()->getTypeIdByHierarchyTypeName(OfdFermaSettingsModel::module, OfdFermaSettingsModel::method));
            if (!$objectId) {
                throw new publicException('Ошибка при создании объекта Настройки');
            }

            $object = $umiObjectsCollection->getObject($objectId);
            if (!$object instanceof umiObject) {
                throw new publicException('Ошибка при получении объекта ' . $objectId);
            }
        }

        $inputData = [
            'object' => $object,
            'allowed-element-types' => [
                OfdFermaSettingsModel::method
            ]
        ];

        if ($this->isSaveMode('param0')) {
            $login = $object->getValue(OfdFermaSettingsModel::field_login);
            $password = $object->getValue(OfdFermaSettingsModel::field_password);

            $object = $this->saveEditedObjectData($inputData);
            if($object instanceof umiObject) {
                if(($login != $object->getValue(OfdFermaSettingsModel::field_login)) || ($password != $object->getValue(OfdFermaSettingsModel::field_password))) {
                    $oMacros = $this->module->getImplementedInstance('OfdFermaMacros');
                    if($oMacros instanceof OfdFermaMacros) {
                        try {
                            $oMacros->getOfdFermaAuthToken(true);
                        } catch (Exception $e) {}
                    }
                }
            }

            $this->chooseRedirect();
        }


        $this->setDataType('form');
        $this->setActionType('modify');
        $data = $this->prepareData($inputData, 'object');
        $this->setData($data);
        $this->doData();
    }

    public function receipt()
    {
        $object = $this->expectObject('param0', true);
        $action = getRequest('param1');

        switch ($action) {
            case 'update': {
                $oMacros = $this->module->getImplementedInstance('OfdFermaMacros');
                if (!$oMacros instanceof OfdFermaMacros) {
                    throw new publicException('Не найден класс OfdFermaMacros');
                }

                $oMacros->updateOfdFermaReceiptStatus($object);

                $this->chooseRedirect();

                break;
            }
        }

        $inputData = [
            'object' => $object,
            'allowed-element-types' => [
                OfdFermaReceiptModel::method
            ]
        ];

        $this->setDataType('form');
        $this->setActionType('modify');
        $data = $this->prepareData($inputData, 'object');
        $this->setData($data);
        $this->doData();
    }

    public function createReceipt()
    {
        $orderId = getRequest('param0');
        if (!$orderId) {
            $this->module->errorNewMessage('Не передан id закаэа');
        }

        $typeId = getRequest('param1');
        if (!$typeId) {
            $this->module->errorNewMessage('Не передан тип чека');
        }

        cmsController::getInstance()->getModule('emarket');

        $order = order::get($orderId);
        if (!$order instanceof order) {
            $this->module->errorNewMessage('Не найден заказ ' . $orderId);
        }

        $oMacros = $this->module->getImplementedInstance('OfdFermaMacros');
        if (!$oMacros instanceof OfdFermaMacros) {
            $this->module->errorNewMessage('Не найден класс OfdFermaMacros');
        }

        try {
            $oMacros->addOfdFermaReceipt($order, $typeId);
        } catch (Exception $e) {
            $this->module->errorNewMessage($e->getMessage());
        }

        $this->chooseRedirect();
    }

    public function receiptLink()
    {
        $receipt = $this->expectObject('param0', true);
        if(!$receipt instanceof umiObject) {
            throw new publicException('Не найден чек');
        }

        $link = $receipt->getValue(OfdFermaReceiptModel::field_link);
        if(!$link) {
            throw new publicException('У чека не заполнено поле "Ссылка"');
        }

        $buffer = outputBuffer::current('HTTPOutputBuffer');
        /* @var $buffer HTTPOutputBuffer*/

        $buffer->redirect($link);
    }

    public function deleteReceipt()
    {
        throw new publicException('Метод не поддерживается');
    }

    public function getDatasetConfiguration($param = '')
    {
        return [
            'methods' => [
                [
                    'title' => getLabel('smc-load'),
                    'forload' => true,
                    'module' => 'ofdFerma',
                    '#__name' => 'receipts'
                ],
                [
                    'title' => getLabel('smc-delete'),
                    'module' => 'ofdFerma',
                    '#__name' => 'deleteReceipt',
                    'aliases' => 'tree_delete_element,delete,del'
                ]
            ],
            'types' => [
                [
                    'common' => 'true',
                    'id' => OfdFermaReceiptModel::method
                ]
            ],
            'stoplist' => [
                OfdFermaReceiptModel::field_link
            ],
            'default' => 'name[50px]|' . OfdFermaReceiptModel::field_id
        ];
    }
}