<?php

/**
 * Created by Evgenii Ioffe
 * @author Evgenii Ioffe <ioffe@umispec.ru>
 * @copyright Copyright (c) 2019, Evgenii Ioffe
 */

use UmiCms\Service;

class ofdFerma extends def_module
{
    public function __construct()
    {
        parent::__construct();

        if (Service::Request()->isAdmin()) {
            $this->includeAdminClasses();

            $commonTabs = $this->getCommonTabs();
            if ($commonTabs instanceof iAdminModuleTabs) {
                $commonTabs->add('receipts', ['receipt']);
                $commonTabs->add('settings');
            }
        }

        $this->includeCommonClasses();
    }

    /**
     * @return umiObject|bool
     * @throws selectorException
     */
    public function getOfdFermaSettingsObject()
    {
        $sel = new selector('objects');
        $sel->types('hierarchy-type')->name(OfdFermaSettingsModel::module, OfdFermaSettingsModel::method);
        $sel->order('id')->asc();

        $result = $sel->result();

        return $result ? $result[0] : false;
    }

    /**
     * @param string $email
     * @return OfdFermaApi
     */
    public function getOfdFermaApi($email = '')
    {
        $api = new OfdFermaApi();

        $email = trim($email);
        if ($email) {
            $api->setEmail($email);
        }
        $api->setLogPath(CURRENT_WORKING_DIR . '/errors/logs/ofdFerma.log');

        return $api;
    }

    public function getObjectEditLink($objectId, $objectType = false)
    {
        $link = false;

        switch($objectType) {
            case OfdFermaReceiptModel::method: {
                $link = '/admin/ofdFerma/receipt/' . $objectId . '/';
                break;
            }
        }

        return $link;
    }

    private function includeAdminClasses()
    {
        $this->__loadLib('admin.php');
        $this->__implement('OfdFermaAdmin');
    }

    private function includeCommonClasses()
    {
        $this->__loadLib('handlers.php');
        $this->__implement('OfdFermaHandlers');

        $this->__loadLib('macros.php');
        $this->__implement('OfdFermaMacros');
    }
}