<?php

/**
 * Created by Evgenii Ioffe
 * @author Evgenii Ioffe <ioffe@umispec.ru>
 * @copyright Copyright (c) 2019, Evgenii Ioffe
 */
class OfdFermaReceiptStatusModel
{
    const module = 'ofdFerma';
    const method = 'ofdFerma_receipt_status';

    const group_properties = 'ofrs_properties';
    const field_code = 'ofrs_code';

    public static function getCodeById($objectId)
    {
        $object = umiObjectsCollection::getInstance()->getObject($objectId);
        if (!$object instanceof umiObject) {
            throw new publicException('Не найден объект ' . $objectId);
        }

        return $object->getValue(self::field_code);
    }

    protected static $arCodeToId = array();

    public static function getIdByCode($code)
    {
        if (!isset(self::$arCodeToId[$code])) {
            $sel = new selector('objects');
            $sel->types('hierarchy-type')->name(self::module, self::method);
            $sel->where(self::field_code)->equals($code);
            $sel->option('return', 'id');
            $sel->order('id')->asc();
            $sel->limit(0, 1);

            $result = $sel->result();

            self::$arCodeToId[$code] = $result ? $result[0]['id'] : false;
        }

        return self::$arCodeToId[$code];
    }

    public static function addCode($code, $name)
    {
        $objectId = self::getIdByCode($code);
        if($objectId) {
            return $objectId;
        }

        $objectTypeId = umiObjectTypesCollection::getInstance()->getTypeIdByHierarchyTypeName(self::module, self::method);

        $objectId = umiObjectsCollection::getInstance()->addObject($name, $objectTypeId);
        if(!$objectId) {
            throw new publicException('Ошибка при создании объекта ' . $objectId);
        }

        $object = umiObjectsCollection::getInstance()->getObject($objectId);
        if(!$object instanceof umiObject) {
            throw new publicException('Ошибка при получении объекта ' . $objectId);
        }

        $object->setValue(self::field_code, $code);
        $object->commit();

        self::$arCodeToId[$code] = $objectId;

        return self::$arCodeToId[$code];
    }
}