<?php

require_once dirname(__FILE__) . '/Field.php';
require_once dirname(__FILE__) . '/Group.php';
require_once dirname(__FILE__) . '/ObjectType.php';
require_once dirname(__FILE__) . '/HierarchyType.php';

/**
 * Created by Evgenii Ioffe
 * @author Evgenii Ioffe <ioffe@umispec.ru>
 * @copyright Copyright (c) 2017, Evgenii Ioffe
 */
class UmiSpecInstaller
{
    const guide_value_guid = 'guid';
    const guide_value_name = 'name';

    /**
     * @var umiHierarchyTypesCollection
     */
    protected $umiHierarchyTypesCollection;

    /**
     * @var umiObjectTypesCollection
     */
    protected $umiObjectTypesCollection;

    /**
     * @var umiFieldsCollection
     */
    protected $umiFieldsCollection;

    /**
     * @var umiFieldTypesCollection
     */
    protected $umiFieldTypesCollection;

    /**
     * @var umiObjectsCollection
     */
    protected $umiObjectsCollection;

    public function __construct()
    {
        $this->umiHierarchyTypesCollection = umiHierarchyTypesCollection::getInstance();
        $this->umiObjectTypesCollection = umiObjectTypesCollection::getInstance();
        $this->umiFieldsCollection = umiFieldsCollection::getInstance();
        $this->umiFieldTypesCollection = umiFieldTypesCollection::getInstance();
        $this->umiObjectsCollection = umiObjectsCollection::getInstance();
    }

    public function getRootGuidesObjectTypeId()
    {
        return $this->umiObjectTypesCollection->getTypeIdByGUID('root-guides-type');
    }

    public function getRootPagesObjectTypeId()
    {
        return $this->umiObjectTypesCollection->getTypeIdByGUID('root-pages-type');
    }

    /**
     * @param $name
     * @param string $method
     * @return bool|umiObjectType
     */
    public function getObjectTypeByHierarchyTypeName($name, $method = '')
    {
        $typeId = method_exists($this->umiObjectTypesCollection, 'getTypeIdByHierarchyTypeName') ? $this->umiObjectTypesCollection->getTypeIdByHierarchyTypeName($name, $method) : $this->umiObjectTypesCollection->getBaseType($name, $method);

        if (!$typeId) {
            return false;
        }

        $type = $this->umiObjectTypesCollection->getType($typeId);

        return $type instanceof umiObjectType ? $type : false;
    }

    public function getObjectTypeByGUID($guid)
    {
        $type = $this->umiObjectTypesCollection->getTypeByGUID($guid);

        return $type instanceof umiObjectType ? $type : false;
    }

    public function getObjectTypeIdByGUID($guid)
    {
        return $this->umiObjectTypesCollection->getTypeIdByGUID($guid);
    }

    public function createObjectTypeByGUID(UmiSpecInstallerObjectType $objectType, $bUpdate = true)
    {
        $guid = $objectType->getGuid();

        if (!$guid) {
            throw new publicException('У класса ' . $objectType->getName() . ' не указан guid');
        }

        $type = $this->getObjectTypeByGUID($guid);

        if (!$type instanceof umiObjectType) {
            $typeId = $this->umiObjectTypesCollection->addType($objectType->getParentId(), $objectType->getName());
            if (!$typeId) {
                throw new publicException('Ошибка при создании класса ' . $objectType->getName());
            }

            $type = $this->umiObjectTypesCollection->getType($typeId);

            if (!$type instanceof umiObjectType) {
                throw new publicException('Ошибка при получении класса ' . $typeId);
            }

            $type->setGUID($objectType->getGuid());
            $type->setIsGuidable($objectType->isIsGuidable());
            $type->commit();

            $this->umiObjectTypesCollection->clearCache();
        } else {
            if ($bUpdate) {
                $type->setName($objectType->getName());
                $type->setIsGuidable($objectType->isIsGuidable());
                $type->commit();
            }
        }

        return $type instanceof umiObjectType ? $type : false;
    }

    public function createObjectTypeByHierarchyTypeId(UmiSpecInstallerObjectType $objectType, $bUpdate = true)
    {
        $hierarchyTypeId = $objectType->getHierarchyTypeId();
        if (!$hierarchyTypeId) {
            throw new publicException('У класса ' . $objectType->getName() . ' не указан иерархический тип данных');
        }

        $type = false;

        $typeId = $this->umiObjectTypesCollection->getTypeIdByHierarchyTypeId($hierarchyTypeId, true);
        if($typeId) {
            $type = $this->umiObjectTypesCollection->getType($typeId);
        }

        if (!$type instanceof umiObjectType) {
            $typeId = $this->umiObjectTypesCollection->addType($objectType->getParentId(), $objectType->getName());
            if (!$typeId) {
                throw new publicException('Ошибка при создании класса ' . $objectType->getName());
            }

            $type = $this->umiObjectTypesCollection->getType($typeId);

            if (!$type instanceof umiObjectType) {
                throw new publicException('Ошибка при получении класса ' . $typeId);
            }

            $type->setHierarchyTypeId($hierarchyTypeId);
            $type->setIsGuidable($objectType->isIsGuidable());
            $type->commit();

            $this->umiObjectTypesCollection->clearCache();
        } else {
            if ($bUpdate) {
                $type->setName($objectType->getName());
                $type->setIsGuidable($objectType->isIsGuidable());
                $type->commit();
            }
        }

        return $type instanceof umiObjectType ? $type : false;
    }

    public function createObjectTypeGroups($arGroups, umiObjectType $type, umiObjectType $baseType = null)
    {
        foreach ($arGroups as $arGroup) {
            $this->createObjectTypeGroup($arGroup, $type, $baseType);
        }

        $subTypesList = $this->umiObjectTypesCollection->getSubTypesList($type->getId());

        if ($subTypesList) {
            foreach ($subTypesList as $subTypeId) {
                $subType = $this->umiObjectTypesCollection->getType($subTypeId);

                if (!$subType instanceof umiObjectType) {
                    continue;
                }

                $this->createObjectTypeGroups($arGroups, $subType, $type);
            }
        }
    }

    public function createObjectTypeGroup(UmiSpecInstallerGroup $oGroup, umiObjectType $type, umiObjectType $baseType = null)
    {
        $name = $oGroup->getName();

        if (!$name) {
            throw new publicException('Не указан идентификатор группы');
        }

        $title = $oGroup->getTitle();
        $active = $oGroup->isActive();
        $visible = $oGroup->isVisible();
        $tip = $oGroup->getTip();

        $group = $type->getFieldsGroupByName($name, true);

        if (!$group instanceof umiFieldsGroup) {
            $groupId = $type->addFieldsGroup($name, $title, $active, $visible, $tip);

            if (!$groupId) {
                throw new publicException('Не удалось создать группу полей ' . $title . ' (' . $name . ') в типе данных ' . $type->getId());
            }

            $group = $type->getFieldsGroup($groupId);

            if (!$group instanceof umiFieldsGroup) {
                throw new publicException('Не удалось создать группу полей ' . $title . ' (' . $name . ') в типе данных ' . $type->getId());
            }
        }

        if ($group->getTitle() != $title) {
            $group->setTitle($title);
        }

        if ($group->getIsActive() != $active) {
            $group->setIsActive($active);
        }

        if ($group->getIsVisible() != $visible) {
            $group->setIsVisible($visible);
        }

        if ($group->getIsLocked()) {
            $group->setIsLocked(false);
        }

        if ($group->getIsUpdated()) {
            $group->commit();
        }

        $fields = $oGroup->getFields();

        if ($fields) {
            foreach ($fields as $arField) {
                $this->createObjectTypeField($arField, $type, $group, $baseType);
            }
        }
    }

    public function createObjectTypeField(UmiSpecInstallerField $oField, umiObjectType $type, umiFieldsGroup $group, umiObjectType $baseType = null)
    {
        $name = $oField->getName();

        if (!$name) {
            throw new publicException('Не указан идентификатор поля');
        }

        $title = $oField->getTitle();
        $typeId = $oField->getTypeId();
        $visible = $oField->isVisible();
        $locked = $oField->isLocked();

        $bAttach = true;

        $fieldId = $type->getFieldId($name);

        if (!$fieldId) {
            if (!$baseType) {
                $fieldId = $this->umiFieldsCollection->addField($name, $title, $typeId, $visible, $locked);

                if (!$fieldId) {
                    throw new publicException('Ошибка при создании поля ' . $title);
                }
            } else {
                $fieldId = $baseType->getFieldId($name);
            }
        } else {
            $bAttach = false;
        }

        if (!$fieldId) {
            throw new publicException('Не найдено поле ' . $name);
        }

        $field = $this->umiFieldsCollection->getField($fieldId);

        if (!$field instanceof umiField) {
            throw new publicException('Не найдено поле ' . $fieldId);
        }

        if ($field->getTitle() != $title) {
            $field->setTitle($title);
        }

        if ($field->getFieldTypeId() != $typeId) {
            $field->setFieldTypeId($typeId);
        }

        if ($field->getIsVisible() != $visible) {
            $field->setIsVisible($visible);
        }

        $field->setImportanceStatus($visible);

        if ($field->getIsLocked() != $locked) {
            $field->setIsLocked($locked);
        }

        $required = $oField->isRequired();

        if ($field->getIsRequired() != $required) {
            $field->setIsRequired($required);
        }

        $tip = $oField->getTip();

        if ($field->getTip() != $tip) {
            $field->setTip($tip);
        }

        $guideId = $oField->getGuideId();

        if ($field->getGuideId() != $guideId) {
            $field->setGuideId($guideId);
        }

        $field->setIsInFilter($oField->isInFilter());

        if (!$field->getIsInheritable()) {
            $field->setIsInheritable(true);
        }

        if ($field->getIsUpdated()) {
            $field->commit();
        }

        if ($bAttach) {
            $group->attachField($fieldId);
        }
    }

    public function getFieldTypeId($type, $isMultiple = false)
    {
        static $types = array();

        $hash = $type;

        if ($isMultiple) {
            $hash .= '_multiple';
        }

        if (!isset($types[$hash])) {
            $fieldType = $this->umiFieldTypesCollection->getFieldTypeByDataType($type, $isMultiple);

            if (!$fieldType instanceof umiFieldType) {
                throw new publicException('Не найдено поле с типом ' . $type);
            }

            $types[$hash] = $fieldType->getId();
        }

        return $types[$hash];
    }

    public function createGuideValues($typeId, $values = array())
    {
        if (!$values) {
            return false;
        }

        $type = $this->umiObjectTypesCollection->getType($typeId);

        if (!$type instanceof umiObjectType) {
            return false;
        }

        $arValues = array();

        foreach ($values as $value) {
            $guid = getArrayKey($value, self::guide_value_guid);

            if (!$guid) {
                continue;
            }

            $arValues[$guid] = getArrayKey($value, self::guide_value_name);
        }

        $sel = new selector('objects');
        $sel->types('object-type')->id($typeId);
        $sel->where('guid')->equals(array_keys($arValues));

        $guideValues = array();

        foreach ($sel as $object) {
            if (!$object instanceof umiObject) {
                continue;
            }

            $guid = $object->getGUID();

            if (!$guid) {
                continue;
            }

            $guideValues[$guid] = $object->getId();
        }

        foreach ($arValues as $guid => $name) {
            if (isset($guideValues[$guid])) {
                $objectId = $guideValues[$guid];
            } else {
                $objectId = $this->umiObjectsCollection->addObject($name, $typeId);
            }

            $object = $this->umiObjectsCollection->getObject($objectId);

            if (!$object instanceof umiObject) {
                continue;
            }

            $object->setGUID($guid);
            $object->setName($name);
            $object->commit();
        }

        return true;
    }

    public function createGuideValueArray($guid, $name)
    {
        return array(
            self::guide_value_guid => $guid,
            self::guide_value_name => $name
        );
    }

    /**
     * @param UmiSpecInstallerHierarchyType $hierarchyType
     * @return umiHierarchyType
     * @throws publicException
     */
    public function createHierarchyType(UmiSpecInstallerHierarchyType $hierarchyType)
    {
        $type = $this->umiHierarchyTypesCollection->getTypeByName($hierarchyType->getModule(), $hierarchyType->getMethod());
        if (!$type instanceof umiHierarchyType) {
            $typeId = $this->umiHierarchyTypesCollection->addType($hierarchyType->getModule(), $hierarchyType->getTitle(), $hierarchyType->getMethod());
            if (!$typeId) {
                throw new publicException('Ошибка при создании иерархического типа данных ' . $hierarchyType->getModule() . ':' . $hierarchyType->getMethod());
            }

            $type = $this->umiHierarchyTypesCollection->getType($typeId);
            if (!$type instanceof umiHierarchyType) {
                throw new publicException('Не найден иерархический тип данных ' . $typeId);
            }

            $this->umiHierarchyTypesCollection->clearCache();
        } else {
            $type->setTitle($hierarchyType->getTitle());
            $type->commit();
        }

        return $type;
    }

    public function createGuideObjectType($name, $guid)
    {
        $objectType = $this->createObjectTypeByGUID(new UmiSpecInstallerObjectType($name, $guid, $this->getRootGuidesObjectTypeId(), true));
        if (!$objectType instanceof umiObjectType) {
            throw new publicException('Ошибка при создании справочника ' . $name);
        }
        $objectType->setIsGuidable(true);
        $objectType->setIsPublic(true);
        $objectType->commit();

        return $objectType;
    }
}