<?php

use UmiCms\Service;

/**
 * Created by Evgenii Ioffe
 * @author Evgenii Ioffe <ioffe@umispec.ru>
 * @copyright Copyright (c) 2019, Evgenii Ioffe
 */
class OfdFermaHandlers
{
    /**
     * @var $module ofdFerma
     */
    public $module;

    public function onStatusChangedOfdFerma(iUmiEventPoint $eventPoint)
    {
        if ($eventPoint->getMode() !== 'after') {
            return false;
        }

        if ($eventPoint->getParam('old-status-id') == $eventPoint->getParam('new-status-id')) {
            return false;
        }

        /** @var order $order */
        $order = $eventPoint->getRef('order', $eventPoint->getParam('new-status-id'));

        $this->orderStatusChangeHandler($order);

        return true;
    }

    public function onModifyObjectOfdFerma(iUmiEventPoint $eventPoint)
    {
        /** @var iUmiObject $object */
        $object = $eventPoint->getRef('object');
        if (!$object instanceof umiObject) {
            return false;
        }

        if ($object->getTypeId() != umiObjectTypesCollection::getInstance()->getTypeIdByHierarchyTypeName('emarket', 'order')) {
            return false;
        }

        static $oldStatusId = null;

        if ($eventPoint->getMode() == 'before') {
            $oldStatusId = $object->getValue('status_id');
        } elseif ($eventPoint->getMode() == 'after') {
            $data = getRequest('data');
            $newStatusId = getArrayKey($data[$object->getId()], 'status_id');

            if ($oldStatusId != $newStatusId) {
                cmsController::getInstance()->getModule('emarket');
                $this->orderStatusChangeHandler(order::get($object->getId()), $newStatusId);
            }
        }

        return true;
    }

    public function onModifyPropertyOfdFerma(iUmiEventPoint $eventPoint)
    {
        if ($eventPoint->getMode() !== 'after') {
            return false;
        }

        if ($eventPoint->getParam('property') != 'status_id') {
            return false;
        }

        /** @var iUmiObject $entity */
        $entity = $eventPoint->getRef('entity');
        if (!$entity instanceof iUmiObject || $entity->getTypeId() != umiObjectTypesCollection::getInstance()->getTypeIdByHierarchyTypeName('emarket', 'order')) {
            return false;
        }

        if ($eventPoint->getParam('oldValue') == $eventPoint->getParam('newValue')) {
            return false;
        }

        cmsController::getInstance()->getModule('emarket');
        $this->orderStatusChangeHandler(order::get($entity->getId()), $eventPoint->getParam('newValue'));

        return true;
    }

    public function onCronOfdFerma(iUmiEventPoint $eventPoint)
    {
        $oMacros = $this->module->getImplementedInstance('OfdFermaMacros');
        if(!$oMacros instanceof OfdFermaMacros) {
            return false;
        }

        try {
            $token = $oMacros->getOfdFermaAuthToken();
        } catch (Exception $e) {
            $token = false;
        }

        if(!$token) {
            return false;
        }

        $sel = new selector('objects');
        $sel->types('object-type')->name(OfdFermaReceiptModel::module, OfdFermaReceiptModel::method);
        $sel->where(OfdFermaReceiptModel::field_state)->isnull(true);
        $sel->order('id')->desc();
        $sel->limit(0, 50);

        foreach($sel as $object) {
            if(!$object instanceof umiObject) {
                continue;
            }

            try {
                $oMacros->updateOfdFermaReceiptStatus($object);
            } catch (Exception $e) {}
        }

        return true;
    }

    protected function orderStatusChangeHandler(order $order, $statusId = false)
    {
        $settings = $this->module->getOfdFermaSettingsObject();
        if (!$settings instanceof umiObject) {
            return false;
        }

        $orderStatusId = $settings->getValue(OfdFermaSettingsModel::field_order_status);
        if (!$orderStatusId) {
            return false;
        }

        if ($statusId != $orderStatusId) {
            return false;
        }

        $oMacros = $this->module->getImplementedInstance('OfdFermaMacros');
        if (!$oMacros instanceof OfdFermaMacros) {
            return false;
        }

        try {
            $oMacros->addOfdFermaReceipt($order, OfdFermaReceiptTypeModel::getIdByCode('Income'));
        } catch (Exception $e) {
        }

        return true;
    }
}
