<?php

/**
 * Created by Evgenii Ioffe
 * @author Evgenii Ioffe <ioffe@umispec.ru>
 * @copyright Copyright (c) 2019, Evgenii Ioffe
 */
class OfdFermaInstaller
{
    /**
     * @var UmiSpecInstaller
     */
    private $installer;

    private $orderObjectTypeId;

    public function __construct()
    {
        require_once __DIR__ . '/classes/vendor/UmiSpec/Installer/Installer.php';

        $this->installer = new UmiSpecInstaller();

        $autoloadFile = __DIR__ . '/autoload.php';

        if (file_exists($autoloadFile)) {
            $classes = [];
            /** @noinspection PhpIncludeInspection */
            require_once $autoloadFile;
            umiAutoload::addClassesToAutoload($classes);
        }

        $this->orderObjectTypeId = umiObjectTypesCollection::getInstance()->getTypeIdByHierarchyTypeName('emarket', 'order');
        if(!$this->orderObjectTypeId) {
            throw new publicException('Не найден тип данных emarket-order');
        }
    }

    public function doInstall()
    {
        $this->doInstallSettingsObjectType();
        $this->doInstallReceiptObjectType();

        $this->doInstallPermissions();
    }

    private function doInstallSettingsObjectType()
    {
        $installerHierarchyType = new UmiSpecInstallerHierarchyType(OfdFermaSettingsModel::module, OfdFermaSettingsModel::method, 'ОФД Ferma: Настройки');

        $hierarchyType = $this->installer->createHierarchyType($installerHierarchyType);
        if (!$hierarchyType instanceof umiHierarchyType) {
            throw new publicException('Ошибка при создании иерархического типа данных ' . $installerHierarchyType->getTitle());
        }

        $installerObjectType = new UmiSpecInstallerObjectType($installerHierarchyType->getTitle(), '', 0);
        $installerObjectType->setHierarchyTypeId($hierarchyType->getId());

        $objectType = $this->installer->createObjectTypeByHierarchyTypeId($installerObjectType);
        if (!$objectType instanceof umiObjectType) {
            throw new publicException('Ошибка при создании объектного типа данных ' . $installerObjectType->getName());
        }

        $group = new UmiSpecInstallerGroup(OfdFermaSettingsModel::group_properties, 'Настройки');
        $field = new UmiSpecInstallerField(OfdFermaSettingsModel::field_login, 'Логин', $this->installer->getFieldTypeId('string'), true);
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaSettingsModel::field_password, 'Пароль', $this->installer->getFieldTypeId('string'), true);
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaSettingsModel::field_inn, 'ИНН', $this->installer->getFieldTypeId('int'), true);
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaSettingsModel::field_taxation, 'Система налогообложения', $this->installer->getFieldTypeId('relation'), true);
        $field->setGuideId($this->doInstallTaxationObjectType());
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaSettingsModel::field_default_vat, 'НДС по умолчанию', $this->installer->getFieldTypeId('relation'));
        $field->setGuideId($this->doInstallVatObjectType());
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaSettingsModel::field_one_line, 'Свёртка в одну строку', $this->installer->getFieldTypeId('boolean'));
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaSettingsModel::field_one_line_label, 'Свёртка в одну строку: Название в чеке', $this->installer->getFieldTypeId('text'));
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaSettingsModel::field_email, 'Адрес e-mail для уведомлений', $this->installer->getFieldTypeId('string'));
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaSettingsModel::field_order_status, 'Статус заказа, при котором отправлять чек', $this->installer->getFieldTypeId('relation'));
        $field->setGuideId($this->installer->getObjectTypeIdByGUID('emarket-orderstatus'));
        $group->addField($field);
        $this->installer->createObjectTypeGroup($group, $objectType);
    }

    private function doInstallTaxationObjectType()
    {
        $installerHierarchyType = new UmiSpecInstallerHierarchyType(OfdFermaTaxationModel::module, OfdFermaTaxationModel::method, 'ОФД Ferma: Система налогообложения');

        $hierarchyType = $this->installer->createHierarchyType($installerHierarchyType);
        if (!$hierarchyType instanceof umiHierarchyType) {
            throw new publicException('Ошибка при создании иерархического типа данных ' . $installerHierarchyType->getTitle());
        }

        $installerObjectType = new UmiSpecInstallerObjectType($installerHierarchyType->getTitle(), '', 0);
        $installerObjectType->setHierarchyTypeId($hierarchyType->getId());

        $objectType = $this->installer->createObjectTypeByHierarchyTypeId($installerObjectType);
        if (!$objectType instanceof umiObjectType) {
            throw new publicException('Ошибка при создании объектного типа данных ' . $installerObjectType->getName());
        }

        $objectType->setIsGuidable(true);
        $objectType->setIsPublic(false);

        $group = new UmiSpecInstallerGroup(OfdFermaTaxationModel::group_properties, 'Свойства');
        $field = new UmiSpecInstallerField(OfdFermaTaxationModel::field_code, 'Код', $this->installer->getFieldTypeId('string'), true);
        $group->addField($field);
        $this->installer->createObjectTypeGroup($group, $objectType);

        $objectTypeId = $objectType->getId();

        $arValues = array(
            'Common' => 'Общая система налогобложения',
            'SimpleIn' => 'Упрощенная система налогообложения (доход)',
            'SimpleInOut' => 'Упрощенная система налогообложения (доход минус расход)',
            'Unified' => 'Единый налог на вмененный доход',
            'UnifiedAgricultural' => 'Единый сельскохозяйственный налог',
            'Patent' => 'Патентная система налогообложения'
        );

        $sel = new selector('objects');
        $sel->types('object-type')->id($objectTypeId);
        $sel->order('id')->asc();

        foreach($sel as $object) {
            if(!$object instanceof umiObject) {
                continue;
            }

            $code = $object->getValue(OfdFermaTaxationModel::field_code);
            if(!isset($arValues[$code])) {
                continue;
            }

            $object->setName($arValues[$code]);
            $object->commit();

            unset($arValues[$code]);
        }

        $umiObjectsCollection = umiObjectsCollection::getInstance();

        foreach($arValues as $code => $name) {
            $objectId = $umiObjectsCollection->addObject($name, $objectTypeId);

            if(!$objectId) {
                throw new publicException('Ошибка при создании объекта ' . $name);
            }

            $object = $umiObjectsCollection->getObject($objectId);
            if(!$object instanceof umiObject) {
                throw new publicException('Ошибка при получении объекта ' . $objectId);
            }

            $object->setValue(OfdFermaTaxationModel::field_code, $code);
            $object->setIsLocked(true);
            $object->commit();
        }

        return $objectTypeId;
    }

    private function doInstallVatObjectType()
    {
        $installerHierarchyType = new UmiSpecInstallerHierarchyType(OfdFermaVatModel::module, OfdFermaVatModel::method, 'ОФД Ferma: НДС');

        $hierarchyType = $this->installer->createHierarchyType($installerHierarchyType);
        if (!$hierarchyType instanceof umiHierarchyType) {
            throw new publicException('Ошибка при создании иерархического типа данных ' . $installerHierarchyType->getTitle());
        }

        $installerObjectType = new UmiSpecInstallerObjectType($installerHierarchyType->getTitle(), '', 0);
        $installerObjectType->setHierarchyTypeId($hierarchyType->getId());

        $objectType = $this->installer->createObjectTypeByHierarchyTypeId($installerObjectType);
        if (!$objectType instanceof umiObjectType) {
            throw new publicException('Ошибка при создании объектного типа данных ' . $installerObjectType->getName());
        }

        $objectType->setIsGuidable(true);
        $objectType->setIsPublic(false);

        $group = new UmiSpecInstallerGroup(OfdFermaVatModel::group_properties, 'Свойства');
        $field = new UmiSpecInstallerField(OfdFermaVatModel::field_code, 'Код', $this->installer->getFieldTypeId('string'), true);
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaVatModel::field_tax_rate, 'Значение в UMI.CMS', $this->installer->getFieldTypeId('relation'), true);
        $taxRateObjectType = $this->installer->getObjectTypeByGUID('tax-rate-guide');
        if(!$taxRateObjectType instanceof umiObjectType) {
            throw new publicException('Не найден српавочник tax-rate-guide');
        }
        $taxRateObjectTypeId = $taxRateObjectType->getId();
        $field->setGuideId($taxRateObjectTypeId);
        $group->addField($field);
        $this->installer->createObjectTypeGroup($group, $objectType);

        $objectTypeId = $objectType->getId();

        $arValues = array(
            'Vat10' => array(
                'name' => 'НДС 10%',
                'tax-rate' => 'tax-rate-27963'
            ),
            'Vat18' => array(
                'name' => 'НДС 18%',
                'tax-rate' => 'tax-rate-vat-18'
            ),
            'Vat20' => array(
                'name' => 'НДС 20%',
                'tax-rate' => 'tax-rate-27964'
            ),
            'Vat0' => array(
                'name' => 'НДС 0%',
                'tax-rate' => 'tax-rate-27962'
            ),
            'VatNo' => array(
                'name' => 'НДС не облагается',
                'tax-rate' => 'tax-rate-27961'
            ),
            'CalculatedVat10110' => array(
                'name' => 'Вычисленный НДС 10% от 110% суммы',
                'tax-rate' => 'tax-rate-27965'
            ),
            'CalculatedVat18118' => array(
                'name' => 'Вычисленный НДС 18% от 118% суммы',
                'tax-rate' => 'tax-rate-vat-18-118'
            ),
            'CalculatedVat20120' => array(
                'name' => 'Вычисленный НДС 20% от 120% суммы',
                'tax-rate' => 'tax-rate-27966'
            )
        );

        $sel = new selector('objects');
        $sel->types('object-type')->id($objectTypeId);
        $sel->order('id')->asc();

        $umiObjectsCollection = umiObjectsCollection::getInstance();

        foreach($sel as $object) {
            if(!$object instanceof umiObject) {
                continue;
            }

            $code = $object->getValue(OfdFermaVatModel::field_code);
            if(!isset($arValues[$code])) {
                continue;
            }

            $name = $arValues[$code]['name'];
            $taxRate = $arValues[$code]['tax-rate'];

            $object->setName($name);

            $taxRateObjectId = $umiObjectsCollection->getObjectIdByGUID($taxRate);
            if(!$taxRateObjectId) {
                $taxRateObjectId = $umiObjectsCollection->addObject($name, $taxRateObjectTypeId);
                if(!$taxRateObjectId) {
                    throw new publicException('Ошибка при создании объекта ' . $name);
                }

                $taxRateObject = $umiObjectsCollection->getObject($taxRateObjectId);
                if(!$taxRateObject instanceof umiObject) {
                    throw new publicException('Ошибка при получении объекта ' . $taxRateObjectId);
                }

                $taxRateObject->setGUID($taxRate);
                $taxRateObject->commit();
            }

            $object->setValue(OfdFermaVatModel::field_tax_rate, $taxRateObjectId);
            $object->commit();

            unset($arValues[$code]);
        }

        foreach($arValues as $code => $arValue) {
            $name = $arValue['name'];
            $taxRate = $arValue['tax-rate'];

            $objectId = $umiObjectsCollection->addObject($name, $objectTypeId);

            if(!$objectId) {
                throw new publicException('Ошибка при создании объекта ' . $name);
            }

            $object = $umiObjectsCollection->getObject($objectId);
            if(!$object instanceof umiObject) {
                throw new publicException('Ошибка при получении объекта ' . $objectId);
            }

            $object->setIsLocked(true);

            $object->setValue(OfdFermaVatModel::field_code, $code);
            $taxRateObjectId = $umiObjectsCollection->getObjectIdByGUID($taxRate);
            if(!$taxRateObjectId) {
                $taxRateObjectId = $umiObjectsCollection->addObject($name, $taxRateObjectTypeId);
                if(!$taxRateObjectId) {
                    throw new publicException('Ошибка при создании объекта ' . $name);
                }

                $taxRateObject = $umiObjectsCollection->getObject($taxRateObjectId);
                if(!$taxRateObject instanceof umiObject) {
                    throw new publicException('Ошибка при получении объекта ' . $taxRateObjectId);
                }

                $taxRateObject->setGUID($taxRate);
                $taxRateObject->commit();
            }

            $object->setValue(OfdFermaVatModel::field_tax_rate, $taxRateObjectId);
            $object->commit();
        }

        return $objectTypeId;
    }

    private function doInstallReceiptObjectType()
    {
        $installerHierarchyType = new UmiSpecInstallerHierarchyType(OfdFermaReceiptModel::module, OfdFermaReceiptModel::method, 'ОФД Ferma: Чек');

        $hierarchyType = $this->installer->createHierarchyType($installerHierarchyType);
        if (!$hierarchyType instanceof umiHierarchyType) {
            throw new publicException('Ошибка при создании иерархического типа данных ' . $installerHierarchyType->getTitle());
        }

        $installerObjectType = new UmiSpecInstallerObjectType($installerHierarchyType->getTitle(), '', 0);
        $installerObjectType->setHierarchyTypeId($hierarchyType->getId());

        $objectType = $this->installer->createObjectTypeByHierarchyTypeId($installerObjectType);
        if (!$objectType instanceof umiObjectType) {
            throw new publicException('Ошибка при создании объектного типа данных ' . $installerObjectType->getName());
        }

        $group = new UmiSpecInstallerGroup(OfdFermaReceiptModel::group_properties, 'Свойства');
        $field = new UmiSpecInstallerField(OfdFermaReceiptModel::field_id, 'Идентификатор', $this->installer->getFieldTypeId('string'));
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaReceiptModel::field_inn, 'ИНН', $this->installer->getFieldTypeId('string'));
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaReceiptModel::field_fn, 'Номер ФН', $this->installer->getFieldTypeId('string'));
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaReceiptModel::field_rnm, 'РНМ', $this->installer->getFieldTypeId('string'));
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaReceiptModel::field_fdn, 'ФДН', $this->installer->getFieldTypeId('string'));
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaReceiptModel::field_fpd, 'ФПД', $this->installer->getFieldTypeId('string'));
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaReceiptModel::field_type, 'Тип чека', $this->installer->getFieldTypeId('relation'), true);
        $field->setGuideId($this->doInstallReceiptTypeObjectType());
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaReceiptModel::field_amount, 'Сумма', $this->installer->getFieldTypeId('float'));
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaReceiptModel::field_order, 'Заказ', $this->installer->getFieldTypeId('relation'), true);
        $field->setGuideId($this->orderObjectTypeId);
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaReceiptModel::field_link, 'Ссылка на чек', $this->installer->getFieldTypeId('string'));
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaReceiptModel::field_create_date, 'Дата создания чека', $this->installer->getFieldTypeId('date'));
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaReceiptModel::field_modified_date, 'Дата обновления данных чека', $this->installer->getFieldTypeId('date'));
        $group->addField($field);
        $field = new UmiSpecInstallerField(OfdFermaReceiptModel::field_state, 'Статус', $this->installer->getFieldTypeId('relation'));
        $field->setGuideId($this->doInstallReceiptStateObjectType());
        $group->addField($field);
        $this->installer->createObjectTypeGroup($group, $objectType);
    }

    private function doInstallReceiptTypeObjectType()
    {
        $installerHierarchyType = new UmiSpecInstallerHierarchyType(OfdFermaReceiptTypeModel::module, OfdFermaReceiptTypeModel::method, 'ОФД Ferma: Тип чека');

        $hierarchyType = $this->installer->createHierarchyType($installerHierarchyType);
        if (!$hierarchyType instanceof umiHierarchyType) {
            throw new publicException('Ошибка при создании иерархического типа данных ' . $installerHierarchyType->getTitle());
        }

        $installerObjectType = new UmiSpecInstallerObjectType($installerHierarchyType->getTitle(), '', 0);
        $installerObjectType->setHierarchyTypeId($hierarchyType->getId());

        $objectType = $this->installer->createObjectTypeByHierarchyTypeId($installerObjectType);
        if (!$objectType instanceof umiObjectType) {
            throw new publicException('Ошибка при создании объектного типа данных ' . $installerObjectType->getName());
        }

        $objectType->setIsGuidable(true);
        $objectType->setIsPublic(false);

        $group = new UmiSpecInstallerGroup(OfdFermaReceiptTypeModel::group_properties, 'Свойства');
        $field = new UmiSpecInstallerField(OfdFermaReceiptTypeModel::field_code, 'Код', $this->installer->getFieldTypeId('string'), true);
        $group->addField($field);
        $this->installer->createObjectTypeGroup($group, $objectType);

        $objectTypeId = $objectType->getId();

        $arValues = array(
            'Income' => 'Получение денежных средств от покупателя',
            'IncomeReturn' => 'Возврат денежных средств, полученных от покупателя',
            'IncomePrepayment' => 'Авансовый платеж от покупателя',
            'IncomeReturnPrepayment' => 'Возврат аванса',
            'IncomeCorrection' => 'Чек коррекции',
            'Expense' => 'Выдача денежных средств покупателю',
            'ExpenseReturn' => 'Возврат денежных средств, выданных покупателю'
        );

        $sel = new selector('objects');
        $sel->types('object-type')->id($objectTypeId);
        $sel->order('id')->asc();

        foreach($sel as $object) {
            if(!$object instanceof umiObject) {
                continue;
            }

            $code = $object->getValue(OfdFermaReceiptTypeModel::field_code);
            if(!isset($arValues[$code])) {
                continue;
            }

            $object->setName($arValues[$code]);
            $object->commit();

            unset($arValues[$code]);
        }

        $umiObjectsCollection = umiObjectsCollection::getInstance();

        foreach($arValues as $code => $name) {
            $objectId = $umiObjectsCollection->addObject($name, $objectTypeId);

            if(!$objectId) {
                throw new publicException('Ошибка при создании объекта ' . $name);
            }

            $object = $umiObjectsCollection->getObject($objectId);
            if(!$object instanceof umiObject) {
                throw new publicException('Ошибка при получении объекта ' . $objectId);
            }

            $object->setValue(OfdFermaReceiptTypeModel::field_code, $code);
            $object->setIsLocked(true);
            $object->commit();
        }

        return $objectTypeId;
    }

    private function doInstallReceiptStateObjectType()
    {
        $installerHierarchyType = new UmiSpecInstallerHierarchyType(OfdFermaReceiptStatusModel::module, OfdFermaReceiptStatusModel::method, 'ОФД Ferma: Статус чека');

        $hierarchyType = $this->installer->createHierarchyType($installerHierarchyType);
        if (!$hierarchyType instanceof umiHierarchyType) {
            throw new publicException('Ошибка при создании иерархического типа данных ' . $installerHierarchyType->getTitle());
        }

        $installerObjectType = new UmiSpecInstallerObjectType($installerHierarchyType->getTitle(), '', 0);
        $installerObjectType->setHierarchyTypeId($hierarchyType->getId());

        $objectType = $this->installer->createObjectTypeByHierarchyTypeId($installerObjectType);
        if (!$objectType instanceof umiObjectType) {
            throw new publicException('Ошибка при создании объектного типа данных ' . $installerObjectType->getName());
        }

        $objectType->setIsGuidable(true);
        $objectType->setIsPublic(false);

        $group = new UmiSpecInstallerGroup(OfdFermaReceiptStatusModel::group_properties, 'Свойства');
        $field = new UmiSpecInstallerField(OfdFermaReceiptStatusModel::field_code, 'Код', $this->installer->getFieldTypeId('string'), true);
        $group->addField($field);
        $this->installer->createObjectTypeGroup($group, $objectType);

        return $objectType->getId();
    }

    private function doInstallEmarketOrderObjectType()
    {
        $objectType = $this->installer->getObjectTypeByHierarchyTypeName('emarket', 'order');
        if (!$objectType instanceof umiObjectType) {
            throw new publicException('Ошибка при получении объектного типа данных emarket-order');
        }

        $group = new UmiSpecInstallerGroup(OfdFermaEmarketOrderModel::group_properties, 'ОФД Ferma');
        $this->installer->createObjectTypeGroup($group, $objectType);
    }

    private function doInstallPermissions()
    {
        $svUserId = UmiCms\Service::SystemUsersPermissions()->getSvUserId();

        $permissions = permissionsCollection::getInstance();
        $permissions->setModulesPermissions($svUserId, 'ofdFerma', 'receipts');
        $permissions->setModulesPermissions($svUserId, 'ofdFerma', 'settings');
    }
}