<?php

use UmiCms\Service;

/**
 * Created by Evgenii Ioffe
 * @author Evgenii Ioffe <ioffe@umispec.ru>
 * @copyright Copyright (c) 2019, Evgenii Ioffe
 */
class OfdFermaMacros
{
    /**
     * @var $module ofdFerma
     */
    public $module;

    public function getOfdFermaAuthToken($bUpdate = false)
    {
        $token = null;

        $cacheFile = CURRENT_WORKING_DIR . '/sys-temp/ofdFerma/token.json';
        if (!$bUpdate) {
            if (file_exists($cacheFile)) {
                $raw = json_decode(file_get_contents($cacheFile), true);

                if (isset($raw['token']) && isset($raw['validTo']) && $raw['validTo'] > time() - 10) {
                    $token = $raw['token'];
                }
            }
        }

        if (!$token) {
            if(file_exists($cacheFile)) {
                unlink($cacheFile);
            }

            $settings = $this->module->getOfdFermaSettingsObject();
            if (!$settings instanceof umiObject) {
                throw new publicException('Не найдены настройки');
            }

            $login = $settings->getValue(OfdFermaSettingsModel::field_login);
            $password = $settings->getValue(OfdFermaSettingsModel::field_password);

            if (!$login || !$password) {
                throw new publicException('Не указаны логин или пароль');
            }

            $api = $this->module->getOfdFermaApi($settings->getValue(OfdFermaSettingsModel::field_email));
            try {
                $tokenObject = $api->getAuthToken($login, $password);

                if ($tokenObject instanceof OfdFermaApiAuthTokenModel) {
                    $token = $tokenObject->getToken();

                    if (!is_dir(dirname($cacheFile))) {
                        mkdir(dirname($cacheFile), 0775, true);
                    }

                    file_put_contents($cacheFile, json_encode(array(
                        'token' => $token,
                        'validTo' => $tokenObject->getDate()
                    )));
                }
            } catch (Exception $e) {
            }
        }

        return $token;
    }

    public function addOfdFermaReceipt(order $order, $typeId)
    {
        $settings = $this->module->getOfdFermaSettingsObject();
        if (!$settings instanceof umiObject) {
            throw new publicException('Не найдены настройки');
        }

        $inn = $settings->getValue(OfdFermaSettingsModel::field_inn);

        $umiObjectsCollection = umiObjectsCollection::getInstance();

        $receipt = new OfdFermaApiReceiptModel();
        $receipt->setInn($inn);
        $receipt->setType(OfdFermaReceiptTypeModel::getCodeById($typeId));

        $date = time();
        $receipt->setDate($date);

        $receipt->setInvoiceId($order->getId());
        $receipt->setTaxationSystem(OfdFermaTaxationModel::getCodeById($settings->getValue(OfdFermaSettingsModel::field_taxation)));

        $customer = $umiObjectsCollection->getObject($order->getCustomerId());
        if (!$customer instanceof umiObject) {
            throw new publicException('Не найдены данные заказчика для заказа ' . $order->getId());
        }

        $emarket = cmsController::getInstance()->getModule('emarket');
        if (!$emarket instanceof emarket) {
            throw new publicException('Не найден модуль emarket');
        }

        $oEmarketNotification = $emarket->getImplementedInstance('EmarketNotification');
        if (!$oEmarketNotification instanceof EmarketNotification) {
            throw new publicException('Не найден класс EmarketNotification');
        }

        $receipt->setPhone($customer->getValue('phone'));
        $receipt->setEmail($oEmarketNotification->getCustomerEmail($order, $customer));

        $defaultVat = OfdFermaVatModel::getCodeById($settings->getValue(OfdFermaSettingsModel::field_default_vat));

        $amount = 0;

        if ($settings->getValue(OfdFermaSettingsModel::field_one_line)) {
            $label = trim($settings->getValue(OfdFermaSettingsModel::field_one_line_label));
            if (!$label) {
                $label = 'Undefined';
            }

            $item = new OfdFermaApiReceiptItemModel();
            $item->setLabel($label);
            $item->setPrice($order->getActualPrice());
            $item->setAmount($order->getActualPrice());
            $item->setQuantity(1);
            $item->setVat($defaultVat);
            $receipt->addItem($item);

            $amount = $order->getActualPrice();
        } else {
            foreach ($order->getItems() as $orderItem) {
                if (!$orderItem instanceof orderItem) {
                    continue;
                }

                $item = new OfdFermaApiReceiptItemModel();
                $item->setLabel($orderItem->getName());
                $item->setPrice($orderItem->getActualPrice());
                $item->setAmount($orderItem->getTotalActualPrice());
                $item->setQuantity($orderItem->getAmount());

                $vat = false;
                $element = $orderItem->getItemElement(true);
                if ($element instanceof umiHierarchyElement) {
                    if ($taxRateId = $element->getValue('tax_rate_id')) {
                        $vat = OfdFermaVatModel::getCodeByTaxRateId($taxRateId);
                    }
                }

                if (!$vat) {
                    $vat = $defaultVat;
                }

                $item->setVat($vat);
                $receipt->addItem($item);

                $amount += $orderItem->getTotalActualPrice();
            }
        }

        $api = $this->module->getOfdFermaApi($settings->getValue(OfdFermaSettingsModel::field_email));
        $receiptId = $api->addReceipt($receipt, $this->getOfdFermaAuthToken());

        if (!$receiptId) {
            throw new publicException('Ошибка при создании объекта Receipt');
        }

        $objectId = $umiObjectsCollection->addObject($receiptId, umiObjectTypesCollection::getInstance()->getTypeIdByHierarchyTypeName(OfdFermaReceiptModel::module, OfdFermaReceiptModel::method));
        if (!$objectId) {
            throw new publicException('Ошибка при создании объекта ' . $receiptId);
        }

        $object = $umiObjectsCollection->getObject($objectId);
        if (!$object instanceof umiObject) {
            throw new publicException('Ошибка при получении объекта ' . $objectId);
        }

        $object->setValue(OfdFermaReceiptModel::field_type, $typeId);
        $object->setValue(OfdFermaReceiptModel::field_id, $receiptId);
        $object->setValue(OfdFermaReceiptModel::field_order, $order->getId());
        $object->setValue(OfdFermaReceiptModel::field_create_date, $date);
        $object->setValue(OfdFermaReceiptModel::field_inn, $inn);
        $object->setValue(OfdFermaReceiptModel::field_amount, $amount);
        $object->commit();

        return $object->getId();
    }

    public function testAddOfdFermaReceipt($orderId, $typeId)
    {
        cmsController::getInstance()->getModule('emarket');

        $order = order::get($orderId);

        return $this->addOfdFermaReceipt($order, $typeId);
    }

    public function updateOfdFermaReceiptStatus(umiObject $object)
    {
        $api = $this->module->getOfdFermaApi();

        $result = $api->getReceiptStatus($object->getValue(OfdFermaReceiptModel::field_id), $this->getOfdFermaAuthToken());
        if (!$result) {
            throw new publicException('Некорректный ответ');
        }

        $object->setValue(OfdFermaReceiptModel::field_fn, $result->getFn());
        $object->setValue(OfdFermaReceiptModel::field_rnm, $result->getRnm());
        $object->setValue(OfdFermaReceiptModel::field_fdn, $result->getFdn());
        $object->setValue(OfdFermaReceiptModel::field_fpd, $result->getFpd());
        $object->setValue(OfdFermaReceiptModel::field_modified_date, $result->getModifiedDate());

        $stateId = OfdFermaReceiptStatusModel::getIdByCode($result->getName());
        if (!$stateId) {
            $stateId = OfdFermaReceiptStatusModel::addCode($result->getName(), $result->getMessage());
        }

        $object->setValue(OfdFermaReceiptModel::field_state, $stateId);

        $link = 'https://ofd.ru/rec/';
        $link .= implode('/', array(
            $object->getValue(OfdFermaReceiptModel::field_inn),
            $object->getValue(OfdFermaReceiptModel::field_rnm),
            $object->getValue(OfdFermaReceiptModel::field_fn),
            $object->getValue(OfdFermaReceiptModel::field_fdn),
            $object->getValue(OfdFermaReceiptModel::field_fpd),
        ));

        $object->setValue(OfdFermaReceiptModel::field_link, $link);

        $object->commit();
    }

    public function testGetReceiptStatus($objectId)
    {
        $object = umiObjectsCollection::getInstance()->getObject($objectId);
        if (!$object instanceof umiObject) {
            return '';
        }

        $api = $this->module->getOfdFermaApi();
        $result = $api->getReceiptStatus($object->getValue(OfdFermaReceiptModel::field_id), $this->getOfdFermaAuthToken());
        var_dump($result);
        die();
    }

    public function onStatusChangedOfdFerma(iUmiEventPoint $eventPoint)
    {
        if ($eventPoint->getMode() !== 'after') {
            return false;
        }

        if ($eventPoint->getParam('old-status-id') == $eventPoint->getParam('new-status-id')) {
            return false;
        }

        $settings = $this->module->getOfdFermaSettingsObject();
        if (!$settings instanceof umiObject) {
            return false;
        }

        $orderStatusId = $settings->getValue(OfdFermaSettingsModel::field_order_status);
        if (!$orderStatusId) {
            return false;
        }

        if ($eventPoint->getParam('new-status-id') != $orderStatusId) {
            return false;
        }

        /** @var order $order */
        $order = $eventPoint->getRef('order');
        try {
            $this->addOfdFermaReceipt($order, OfdFermaReceiptTypeModel::getIdByCode('Income'));
        } catch (Exception $e) {
        }

        return true;
    }

    public function testOrderStatusChanged($orderId)
    {
        cmsController::getInstance()->getModule('emarket');

        $order = order::get($orderId);

        $eventPoint = new umiEventPoint('order-status-changed');
        $eventPoint->addRef('order', $order);
        $eventPoint->setMode('after');
        $eventPoint->setParam('old-status-id', false);
        $eventPoint->setParam('new-status-id', order::getStatusByCode('waiting'));
        $eventPoint->call();
    }

    public function getOfdFermaOrderReceipts($orderId = false)
    {
        if (!$orderId) {
            throw new publicException('Не передан id заказа');
        }

        $sel = new selector('objects');
        $sel->types('hierarchy-type')->name(OfdFermaReceiptModel::module, OfdFermaReceiptModel::method);
        $sel->where(OfdFermaReceiptModel::field_order)->equals($orderId);
        $sel->order('id')->desc();

        $umiObjectsCollection = umiObjectsCollection::getInstance();

        $items = array();
        foreach ($sel as $object) {
            if (!$object instanceof umiObject) {
                continue;
            }

            $item = array(
                'attribute:id' => $object->getId(),
                'id' => $object->getName(),
                'date_create' => $object->getValue(OfdFermaReceiptModel::field_create_date),
                'date_modified' => $object->getValue(OfdFermaReceiptModel::field_modified_date),
            );

            $typeId = $object->getValue(OfdFermaReceiptModel::field_type);
            if ($typeId) {
                $type = $umiObjectsCollection->getObject($typeId);
                if ($type instanceof umiObject) {
                    $item['type'] = $type->getName();
                }
            }

            $items[] = $item;
        }

        if (!count($items)) {
            return null;

        }
        return array('items' => array('nodes:item' => $items));
    }

    public function getOfdFermaReceiptTypes()
    {
        $sel = new selector('objects');
        $sel->types('hierarchy-type')->name(OfdFermaReceiptTypeModel::module, OfdFermaReceiptTypeModel::method);
        $sel->where(OfdFermaReceiptTypeModel::field_code)->isnotnull(true);
        $sel->order('id')->asc();

        $items = array();
        foreach ($sel as $object) {
            if (!$object instanceof umiObject) {
                continue;
            }

            $code = $object->getValue(OfdFermaReceiptTypeModel::field_code);
            if(!$code) {
                continue;
            }

            $items[] = array(
                'attribute:id' => $object->getId(),
                'attribute:code' => $code,
                'node:text' => $object->getName(),
            );
        }

        if (!count($items)) {
            return null;
        }

        return array('items' => array('nodes:item' => $items));
    }

    public function getOfdFermaReceiptsFilterData()
    {
        $objectTypeId = umiObjectTypesCollection::getInstance()->getTypeIdByHierarchyTypeName(OfdFermaReceiptModel::module, OfdFermaReceiptModel::method);
        if(!$objectTypeId) {
            throw new publicException('Не найден тип данных ' . OfdFermaReceiptModel::module . ':' . OfdFermaReceiptModel::method);
        }

        $objectType = umiObjectTypesCollection::getInstance()->getType($objectTypeId);
        if (!$objectType instanceof umiObjectType) {
            throw new publicException('Не найден тип данных');
        }

        $requestFilter = getRequest('filter');

        $arFields = array(
            OfdFermaReceiptModel::field_create_date,
            OfdFermaReceiptModel::field_type,
            OfdFermaReceiptModel::field_order,
        );

        $umiObjectsCollection = umiObjectsCollection::getInstance();
        $umiFieldsCollection = umiFieldsCollection::getInstance();

        $items = array();
        foreach ($arFields as $fieldName) {
            $fieldId = $objectType->getFieldId($fieldName, false);
            if (!$fieldId) {
                continue;
            }

            $field = $umiFieldsCollection->getField($fieldId);
            if (!$field instanceof umiField) {
                continue;
            }

            $type = $field->getDataType();
            $title = $field->getTitle();

            if($fieldName == OfdFermaReceiptModel::field_order) {
                $type = 'string';
                $title = 'Номер заказа';
            }

            $arField = array(
                'attribute:name' => $fieldName,
                'attribute:title' => $title,
                'attribute:type' => $type
            );

            switch ($type) {
                case "relation": {
                    $selected = getArrayKey($requestFilter, $fieldName);

                    $values = $umiObjectsCollection->getGuidedItems($field->getGuideId());
                    if (!$values) {
                        $values = array();

                        $sel = new selector('objects');
                        $sel->types('object-type')->id($field->getGuideId());
                        $sel->option('return', array('id', 'name'));
                        foreach ($sel as $row) {
                            $values[$row['id']] = $row['name'];
                        }
                    }

                    $arValues = array();
                    foreach ($values as $id => $value) {
                        $item = array(
                            'attribute:id' => $id,
                            'node:value' => $value
                        );

                        if ($id == $selected) {
                            $item['attribute:selected'] = 'selected';
                        }

                        $arValues[] = $item;
                    }

                    $arField['values'] = array('nodes:item' => $arValues);
                    break;
                }
                case "date": {
                    $values = getArrayKey($requestFilter, $fieldName);

                    if (is_array($values) && isset($values['date'])) {
                        $arField['value'] = array(
                            'from' => getArrayKey($values['date'], 'from'),
                            'to' => getArrayKey($values['date'], 'to')
                        );
                    }
                    break;
                }
                case "string": {
                    $arField['value'] = getArrayKey($requestFilter, $fieldName);
                    break;
                }
                default: {
                    break;
                }
            }

            $items[] = $arField;
        }

        if (!count($items)) {
            return null;
        }

        return array('fields' => array('nodes:field' => $items));
    }
}
